/* Anachron theme engine
 * Copyright (C) 2005 Dennis Möhlmann
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Based on Redmont95
 * Written by Owen Taylor <otaylor@redhat.com>
 * Modified by Dennis Möhlmann <nyda at nyda dot info>
 *   possibly importing small code portions from clearlooks engine
 */

#include <math.h>
#include <string.h>
#include <gtk/gtk.h>

#include "config.c"
#include "style.h"
#include "support.c"


/*
 * draw an arrow element (used in scrollbars, spinbuttons etc..)
 */
static void draw_arrow( GtkStyle      *style,
                        GdkWindow     *window,
                        GtkStateType   state,
                        GtkShadowType  shadow,
                        GdkRectangle  *area,
                        GtkWidget     *widget,
                        const gchar   *detail,
                        GtkArrowType   arrow_type,
                        gboolean       fill,
                        gint           x,
                        gint           y,
                        gint           width,
                        gint           height)
{
  sanitize_size(window, &width, &height);

  /* draw small spinbutton arrow */
  if (DETAIL("spinbutton")) {
    x += (width - 7) / 2;

    if (arrow_type == GTK_ARROW_UP)
      y += (height - 4) / 2;
    else
      y += (1 + height - 4) / 2;

    ac_draw_varrow(window, style->fg_gc[state], shadow, area, arrow_type, x, y, 7, 4);
  }

  /* draw scrollbar arrows (normal size) */
  else if (DETAIL("vscrollbar"))
    ac_draw_varrow(window, style->fg_gc[state], shadow, area, arrow_type, x, y, 7, 5);
      
  else if (DETAIL("hscrollbar"))
    ac_draw_harrow (window, style->fg_gc[state], shadow, area, arrow_type, x, y, 5, 7);

  /* various arrows */
  else {
    if (arrow_type == GTK_ARROW_UP || arrow_type == GTK_ARROW_DOWN) {
      x+= (width - 7) / 2;
      y+= (height - 5) / 2;
	  
      ac_draw_varrow (window, style->fg_gc[state], shadow, area, arrow_type, x, y, 7, 5);
  	}
    else {
      x+= (width - 5) / 2;
      y+= (height - 7) / 2;

      ac_draw_harrow (window, style->fg_gc[state], shadow, area, arrow_type, x, y, 5, 7);
    }
  }
}


/*
 * draw various types of boxes (small buttons, large buttons, trough...)
 */
static void draw_box( GtkStyle      *style,
                      GdkWindow     *window,
                      GtkStateType   state_type,
                      GtkShadowType  shadow_type,
                      GdkRectangle  *area,
                      GtkWidget     *widget,
                      const gchar   *detail,
                      gint           x,
                      gint           y,
                      gint           width,
                      gint           height)
{
  GdkGC *gc=    ANACHRON_STYLE(style)->gc;
  GdkGCValues   old_values;

  DEBUGOUT(detail, widget);

  /* make sure size is properly specified, backup old gc color */
	g_return_if_fail(GTK_IS_STYLE (style));
	g_return_if_fail(window != NULL);
  g_return_if_fail(widget != NULL);
  sanitize_size(window, &width, &height);
	gdk_gc_get_values(gc, &old_values);
  if (area)
    gdk_gc_set_clip_rectangle(gc, area);

  /* scrollbar elements */
  if (DETAIL("slider") || DETAIL("hscrollbar") || DETAIL("vscrollbar"))
    ac_draw_box_small(window, style, state_type, x, y, width, height, 3, 5, state_type == GTK_STATE_ACTIVE, GTK_IS_HSCROLLBAR(widget));

  /* v/h scale sliders */
  else if (DETAIL("vscale"))
    ac_draw_box_small(window, style, state_type, x, y, width, height, 3, 5, FALSE, FALSE);
  else if (DETAIL("hscale"))
    ac_draw_box_small(window, style, state_type, x, y, width, height, 3, 5, FALSE, TRUE);

  /* scroll- and progressbar trough */
	else if (DETAIL("trough"))
    ac_draw_box_small(window, style, state_type, x, y, width, height, 6, 5, TRUE, GTK_IS_HSCROLLBAR(widget));

  /* progress bar indicator hack, make smaller than trough (doesn't seem to work from rc-file) */
  else if (GTK_IS_PROGRESS_BAR(widget))
    ac_draw_box_small(window, style, state_type, x +1, y +1, width -2, height -2, 3, 5, FALSE, TRUE);

  /* toolbar: thin line at bottom */
  else if (DETAIL("toolbar"))
    ac_draw_line(window, style, state_type, x, y +height -2, width, TRUE);

  /* toolbar button: reduce size to prevent overlapping with toolbar line */
  else if (DETAIL("button") && trace_root(widget, "GtkToolbar", 5)) // some buttons are nested in multiple h/vboxes in a toolbar
    ac_draw_box_large(window, style, state_type, x, y +1, width -1, height -3, 3, 5, state_type == GTK_STATE_ACTIVE, TRUE);

  /* dock around toolbar */
  else if (DETAIL("handlebox_bin") || DETAIL("dockitem_bin"))
    /* suppressed */;

  /* menubar: sunken */
  else if (GTK_IS_MENU_BAR(widget))
    ac_draw_box_small(window, style, state_type, x, y, width, height, 6, 5, TRUE, TRUE);

  /* menu: flat shaded since it's overriden by GTK anyways, for whatever reason (FIXME: why? where?) */
  else if (GTK_IS_MENU(widget))
    ac_draw_box_small(window, style, state_type, x, y, width, height, 4, 4, FALSE, TRUE);

  /* menuitem */
  else if (DETAIL("menuitem"))
    ac_draw_box_small(window, style, state_type, x, y, width, height, 3, 5,  FALSE, TRUE);

  /* various elements that should draw as small buttons */
	else if ((widget->parent && GTK_IS_TREE_VIEW(widget->parent)) ||
           (DETAIL("spinbutton") || DETAIL("spinbutton_up") || DETAIL("spinbutton_down")	|| DETAIL("menuitem")) )
    ac_draw_box_small(window, style, state_type, x, y, width, height, 3, 5, state_type == GTK_STATE_ACTIVE, TRUE);

  /* default-button shadow */
  else if (DETAIL("buttondefault"))
    ; /* suppressed */ //ac_draw_shadow_large(window, style, x, y, width, height, state_type == GTK_STATE_ACTIVE);

  /* for now everything else */
  else
    ac_draw_box_large(window, style, state_type, x, y, width, height, 3, 5, state_type == GTK_STATE_ACTIVE, TRUE);

  /* restore previous gc */
	gdk_gc_set_foreground (gc, &old_values.foreground);
  if (area)
    gdk_gc_set_clip_rectangle(gc, NULL);
}


/*
 * draw notebook tabs
 */
static void draw_extension( GtkStyle       *style,
                            GdkWindow      *window,
                            GtkStateType    state_type,
                            GtkShadowType   shadow_type,
                            GdkRectangle   *area,
                            GtkWidget      *widget,
                            const gchar    *detail,
                            gint            x,
                            gint            y,
                            gint            width,
                            gint            height,
                            GtkPositionType gap_side)
{
  AnachronStyle *ac_style= ANACHRON_STYLE(style);
  GdkGCValues    old_values;

  DEBUGOUT(detail, widget);

  /* make sure size is properly specified, backup old gc color */
	g_return_if_fail(GTK_IS_STYLE (style));
	g_return_if_fail(window != NULL);
  sanitize_size(window, &width, &height);
	gdk_gc_get_values(ac_style->gc, &old_values);
	if (area)
		gdk_gc_set_clip_rectangle(ac_style->gc, area);

  /* draw only tabs, hand on all other stuff to parent */
  if (DETAIL("tab")) {
    ac_draw_notebook_tab(window, style, state_type, x, y, width, height, FALSE, TRUE);
  }
  else
		parent_class->draw_extension (style, window, state_type, shadow_type, area,
		                              widget, detail, x, y, width, height,
		                              gap_side);

  /* restore previous gc color */
	gdk_gc_set_foreground (ac_style->gc, &old_values.foreground);
	if (area)
		gdk_gc_set_clip_rectangle(ac_style->gc, NULL);
}


/*
 * draws a horizontal line (used in menus or standalone)
 */
static void draw_hline( GtkStyle     *style,
                        GdkWindow    *window,
                        GtkStateType  state_type,
                        GdkRectangle  *area,
                        GtkWidget     *widget,
                        const gchar   *detail,
                        gint          x1,
                        gint          x2,
                        gint          y)
{
	AnachronStyle *ac_style= ANACHRON_STYLE(style);
	GdkGCValues    old_values;

  /* make sure size is properly specified, backup old gc color */
	g_return_if_fail(GTK_IS_STYLE (style));
	g_return_if_fail(window != NULL);
	gdk_gc_get_values(ac_style->gc, &old_values);
	if (area)
		gdk_gc_set_clip_rectangle(ac_style->gc, area);
	
  ac_draw_line(window, style, state_type, x1, y, x2 -x1, TRUE);
	
  /* restore previous gc color */
	gdk_gc_set_foreground (ac_style->gc, &old_values.foreground);
	if (area)
		gdk_gc_set_clip_rectangle (ac_style->gc, NULL);
}


/*
 * draws a vertical line (who would have guessed)
 */
static void draw_vline( GtkStyle     *style,
                        GdkWindow    *window,
                        GtkStateType  state_type,
                        GdkRectangle  *area,
                        GtkWidget     *widget,
                        const gchar   *detail,
                        gint          y1,
                        gint          y2,
                        gint          x)
{
	AnachronStyle *ac_style= ANACHRON_STYLE(style);
	GdkGCValues    old_values;

  /* make sure size is properly specified, backup old gc color */
	g_return_if_fail(GTK_IS_STYLE (style));
	g_return_if_fail(window != NULL);
	gdk_gc_get_values(ac_style->gc, &old_values);
	if (area)
		gdk_gc_set_clip_rectangle(ac_style->gc, area);
	
  ac_draw_line(window, style, state_type, x, y1, y2 -y1, FALSE);
	
  /* restore previous gc */
	gdk_gc_set_foreground(ac_style->gc, &old_values.foreground);
	if (area)
		gdk_gc_set_clip_rectangle(ac_style->gc, NULL);
}


/*
 * draw handle (handle to move toolbars around)
 */
static void draw_handle(GtkStyle       *style,
                        GdkWindow      *window,
                        GtkStateType    state_type,
                        GtkShadowType   shadow_type,
                        GdkRectangle   *area,
                        GtkWidget      *widget,
                        const gchar    *detail,
                        gint            x,
                        gint            y,
                        gint            width,
                        gint            height,
                        GtkOrientation  orientation)
{
	AnachronStyle *ac_style= ANACHRON_STYLE(style);
	GdkGCValues    old_values;
  gint           i, bumps, pos_a, pos_b;

  DEBUGOUT(detail, widget);

  /* make sure size is properly specified, backup old gc color */
	g_return_if_fail(GTK_IS_STYLE (style));
	g_return_if_fail(window != NULL);
	gdk_gc_get_values(ac_style->gc, &old_values);
	if (area)
		gdk_gc_set_clip_rectangle(ac_style->gc, area);
	
  /* Work around orientation bugs */
	if (DETAIL("dockitem") || (widget && strcmp (g_type_name(G_TYPE_FROM_INSTANCE(widget)), "PanelAppletFrame") == 0))	{
		if (orientation == GTK_ORIENTATION_VERTICAL)
			orientation= GTK_ORIENTATION_HORIZONTAL;
		else
			orientation= GTK_ORIENTATION_VERTICAL;
	}

  /* vertical handle: draw some (horizontal!) bumps */
  if (orientation == GTK_ORIENTATION_VERTICAL) {
    bumps= height / 11;
    if (bumps < 3)
      bumps= 3;
    pos_a= (height / 2) -((bumps * 5) / 2);
    pos_b= (width / 2) -(BUMP_LEN / 2);

    for (i= 0; i < bumps; i++)
      ac_draw_bump(window, style, state_type, x +pos_b, y +pos_a +(i * 5), BUMP_LEN, TRUE);

    /* complement toobar underlining, but exclude h/v-paned and gnome panel */
    if (!DETAIL("paned") && widget && strcmp(g_type_name(G_TYPE_FROM_INSTANCE(widget)), "PanelAppletFrame") != 0)
      ac_draw_line(window, style, state_type, x, y +height -2, width, TRUE);
  }

  /* horizontal handle */
  else {
    bumps= width / 11;
    if (bumps < 3)
      bumps= 3;
    pos_a= (width / 2) -((bumps * 5) / 2);
    pos_b= (height / 2) -(BUMP_LEN / 2);

    for (i= 0; i < bumps; i++)
      ac_draw_bump(window, style, state_type, x +pos_a +(i * 5), y +pos_b, BUMP_LEN, TRUE);

    if (!DETAIL("paned") && widget && strcmp(g_type_name(G_TYPE_FROM_INSTANCE(widget)), "PanelAppletFrame") != 0)
      ac_draw_line(window, style, state_type, x +width -2, y, height, TRUE);
  }
	
  /* restore previous gc */
	gdk_gc_set_foreground(ac_style->gc, &old_values.foreground);
	if (area)
		gdk_gc_set_clip_rectangle(ac_style->gc, NULL);
}


/*
 * draw shadow (used in status bar, editboxes...)
 */
static void draw_shadow(GtkStyle       *style,
                        GdkWindow      *window,
                        GtkStateType    state_type,
                        GtkShadowType   shadow_type,
                        GdkRectangle   *area,
                        GtkWidget      *widget,
                        const gchar    *detail,
                        gint            x,
                        gint            y,
                        gint            width,
                        gint            height)
{
  AnachronStyle *ac_style= ANACHRON_STYLE(style);
  gboolean       false_size= FALSE;
  GdkGCValues    old_values;

  DEBUGOUT(detail, widget);

  /* make sure size is properly specified, backup old gc color */
	g_return_if_fail(GTK_IS_STYLE (style));
	g_return_if_fail(window != NULL);
  false_size= sanitize_size(window, &width, &height);
	gdk_gc_get_values(ac_style->gc, &old_values);
	if (area)
		gdk_gc_set_clip_rectangle(ac_style->gc, area);

  /* shadow looks like a small, pressed button */
  ac_draw_box_small(window, style, state_type, x, y, width, height, 2, 3, TRUE, TRUE);

  /* restore previous gc */
	gdk_gc_set_foreground (ac_style->gc, &old_values.foreground);
	if (area)
		gdk_gc_set_clip_rectangle(ac_style->gc, NULL);
}


/*
 * draw shadow gap FIXME: for what?
 */
static void draw_shadow_gap(GtkStyle       *style,
                            GdkWindow      *window,
                            GtkStateType    state_type,
                            GtkShadowType   shadow_type,
                            GdkRectangle   *area,
                            GtkWidget      *widget,
                            const gchar    *detail,
                            gint            x,
                            gint            y,
                            gint            width,
                            gint            height,
                            GtkPositionType gap_side,
                            gint            gap_x,
                            gint            gap_width)
{
  DEBUGOUT(detail, widget);

}


/* doesn't seem to be used? */
static void draw_tab( GtkStyle   *style,
	  GdkWindow     *window,
	  GtkStateType   state,
	  GtkShadowType  shadow,
	  GdkRectangle  *area,
	  GtkWidget     *widget,
	  const gchar   *detail,
	  gint           x,
	  gint           y,
	  gint           width,
	  gint           height)
{
  GtkRequisition indicator_size;
  GtkBorder indicator_spacing;
  gint arrow_height;
  
  DEBUGOUT(detail, widget);

  g_return_if_fail (style != NULL);
  g_return_if_fail (window != NULL);

  if (widget)
    gtk_widget_style_get (widget, "indicator_size", &indicator_size, NULL);

  option_menu_get_props (widget, &indicator_size, &indicator_spacing);

  x += (width - indicator_size.width) / 2;
  arrow_height = (indicator_size.width + 1) / 2;
  
  y += (height - arrow_height) / 2;

  ac_draw_varrow (window, style->black_gc, shadow, area, GTK_ARROW_DOWN,
	       x, y, indicator_size.width, arrow_height);
}


/*
 * initialize style config from resourcefile
 */
static void anachron_style_init_from_rc(GtkStyle *style, GtkRcStyle *rc_style) {
	AnachronStyle *anachron_style= ANACHRON_STYLE(style);
	int i, j;
	
	parent_class->init_from_rc(style, rc_style);
	
  /* create selected shades from the original ana-theme, major field 6 is special as it reuses "normal" for a shadow-palette */
  for (j= 0; j < NUM_STATES; j++)
  	for (i= 0; i < NUM_SHADES; i++)
      adjust_brightness(&anachron_style->border_shades[j][i], &style->bg[j], border_shade_factors[j][i]);
}


/*
 * realize anachron style (set up some color-gcs)
 */
static void anachron_style_realize (GtkStyle *style) {
  AnachronStyle *anachron_style= ANACHRON_STYLE(style);
//  int i;

  parent_class->realize(style);

  /* some default gc to work with */
  anachron_style->gc= create_gc_from_color(style, &anachron_style->border_shades[0][0]);

  /* shaded border colors */
/*
	for (i= 0; i < 10; i++)
		anachron_style->border_shades_gc[i]= create_gc_from_color(style, &anachron_style->border_shades[i]);*/
}


/*
 * unrealize anachron style, deallocate resources
 */
static void anachron_style_unrealize(GtkStyle *style) {
/*  AnachronStyle *anachron_style= ANACHRON_STYLE(style);
  int i;
	
  for (i=0; i < 10; i++)
    gtk_gc_release(anachron_style->border_shades_gc[i]);
*/	
  parent_class->unrealize(style);
}


/*
 * initialize anachron style (dummy)
 */
static void anachron_style_init(AnachronStyle *style) {

}


/*
 * initialize anachron style class (bind methods)
 */
static void anachron_style_class_init(AnachronStyleClass *klass) {
  GtkStyleClass *style_class = GTK_STYLE_CLASS (klass);

  parent_class= g_type_class_peek_parent (klass);

  style_class->realize=         anachron_style_realize;
  style_class->unrealize=       anachron_style_unrealize;
  style_class->init_from_rc=    anachron_style_init_from_rc;

  style_class->draw_arrow=      draw_arrow;
	style_class->draw_handle=     draw_handle;
  style_class->draw_box=        draw_box;
	style_class->draw_hline=      draw_hline;
	style_class->draw_vline=      draw_vline;
  style_class->draw_check=      draw_check;
  style_class->draw_option=     draw_option;
  style_class->draw_tab=        draw_tab;
	style_class->draw_shadow=     draw_shadow;
	style_class->draw_shadow_gap= draw_shadow_gap;
	style_class->draw_extension=  draw_extension;
}


/*
 * register new anachron style type with gtk
 */
GType anachron_type_style= 0;

void anachron_style_register_type(GTypeModule *module) {
  static const GTypeInfo object_info= {
    sizeof(AnachronStyleClass),
    (GBaseInitFunc) NULL,
    (GBaseFinalizeFunc) NULL,
    (GClassInitFunc) anachron_style_class_init,
    NULL,           /* class_finalize */
    NULL,           /* class_data */
    sizeof(AnachronStyle),
    0,              /* n_preallocs */
    (GInstanceInitFunc)anachron_style_init,
  };
  
  anachron_type_style= g_type_module_register_type(module, GTK_TYPE_STYLE, "AnachronStyle", &object_info, 0);
}
