/* Anachron theme engine
 * Copyright (C) 2005 Dennis Möhlmann
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Based on Redmont95
 * Written by Owen Taylor <otaylor@redhat.com>
 * Modified by Dennis Möhlmann <nyda at nyda dot info>
 */

/*
 * some rather helpful debug statements
 */
#if DEBUG
#include <stdio.h>
  #define DEBUGOUT(detail, widget) { \
    if (detail) \
      printf("%s: %s, ", __func__, detail); \
    if (widget) { \
      printf("class: %s, ", g_type_name(G_TYPE_FROM_INSTANCE(widget))); \
      if (widget->parent) { \
        printf("p1: %s ", g_type_name(G_TYPE_FROM_INSTANCE(widget->parent))); \
        if (widget->parent->parent) { \
          printf("p2: %s ", g_type_name(G_TYPE_FROM_INSTANCE(widget->parent->parent))); \
          if (widget->parent->parent->parent) \
            printf("p3: %s", g_type_name(G_TYPE_FROM_INSTANCE(widget->parent->parent->parent))); \
        } \
      } \
    } \
    printf("\n"); \
  }
#else
  #define DEBUGOUT(detail, widget) ((void)0)
#endif


/* Default values, not normally used
 */
static GtkRequisition default_option_indicator_size = { 9, 8 };
static GtkBorder default_option_indicator_spacing = { 7, 5, 2, 2 };

static GtkStyleClass *parent_class;

typedef enum {
  CHECK_AA,
  CHECK_BASE,
  CHECK_BLACK,
  CHECK_DARK,
  CHECK_LIGHT,
  CHECK_MID,
  CHECK_TEXT,
  RADIO_BASE,
  RADIO_BLACK,
  RADIO_DARK,
  RADIO_LIGHT,
  RADIO_MID,
  RADIO_TEXT
} Part;



static struct {
  char      *bits;
  GdkBitmap *bmap;
} parts[] = {
  { check_aa_bits, NULL },
  { check_base_bits, NULL },
  { check_black_bits, NULL },
  { check_dark_bits, NULL },
  { check_light_bits, NULL },
  { check_mid_bits, NULL },
  { check_text_bits, NULL },
  { radio_base_bits, NULL },
  { radio_black_bits, NULL },
  { radio_dark_bits, NULL },
  { radio_light_bits, NULL },
  { radio_mid_bits, NULL },
  { radio_text_bits, NULL }
};


/*
 * check detailed description for a widget
 */
#define DETAIL(xx)   ((detail) && (!strcmp(xx, detail)))


/*
 * trace a widgets roots for a specific type of ancestor (used to find toolbar siblings)
 */
static gboolean trace_root(GtkWidget *widget, gchar *typename, gint depth) {
  gint i;

  for (i= 0; i < depth; i++)
    if (!widget)
      return FALSE;
    else if (strcmp(g_type_name(G_TYPE_FROM_INSTANCE(widget)), typename) == 0)
      return TRUE;
    else
      widget= widget->parent;

  return FALSE;
}


/*
 * size is not properly reported for objects larger than the screen
 */
static gboolean sanitize_size(GdkWindow *window, gint *width, gint *height) {
  gboolean set_bg = FALSE;

  if ((*width == -1) && (*height == -1)) {
    set_bg = GDK_IS_WINDOW (window);
    gdk_window_get_size (window, width, height);
  }
  else if (*width == -1)
    gdk_window_get_size (window, width, NULL);
  else if (*height == -1)
    gdk_window_get_size (window, NULL, height);

  return set_bg;
}


/*
 * create a new graphics context for the specified color
 */
static GdkGC *create_gc_from_color(GtkStyle *style, GdkColor *color) {
	GdkGCValues gc_values;
	
	gdk_colormap_alloc_color(style->colormap, color, FALSE, TRUE);
	
	gc_values.foreground= *color;
	
	return gtk_gc_get(style->depth, style->colormap, &gc_values, GDK_GC_FOREGROUND);
}


/*
 * adjust brightness of a given color
 */
static void adjust_brightness(GdkColor *result, GdkColor *base, float factor) {
  gdouble   red;
  gdouble   green;
  gdouble   blue;
	
  red=    (gdouble)base->red * factor;
  green=  (gdouble)base->green * factor;
  blue=   (gdouble)base->blue * factor;
	
  if (red > COLOR_CLAMP) red= COLOR_CLAMP;
  if (green > COLOR_CLAMP) green= COLOR_CLAMP;
  if (blue > COLOR_CLAMP) blue= COLOR_CLAMP;
  if (red < 0) red= 0;
  if (green < 0) green= 0;
  if (blue < 0) blue= 0;

	result->red=   (guint16)red;
	result->green= (guint16)green;
	result->blue=  (guint16)blue;

}


/*
 * translate state types to color-array
 */
static GdkColor *get_state_palette(GtkStyle *style, GtkStateType state_type) {
  AnachronStyle *ac_style= ANACHRON_STYLE(style);

  switch (state_type) {
    case GTK_STATE_NORMAL:
      return ac_style->border_shades[0];
    case GTK_STATE_ACTIVE:
      return ac_style->border_shades[1];
    case GTK_STATE_PRELIGHT:
      return ac_style->border_shades[2];
    case GTK_STATE_SELECTED:
      return ac_style->border_shades[3];
    case GTK_STATE_INSENSITIVE:
      return ac_style->border_shades[4];
    case 5:
      return ac_style->border_shades[5];
    default: /* normal */
      return ac_style->border_shades[0];
  }    
}


/*
 * draw a gradient
 */
static void ac_draw_gradient( GdkDrawable   *drawable,
                              GdkGC         *gc,
                              gint          x,
                              gint          y,
                              gint          width,
                              gint          height,
                              gboolean      horizontal,
                              GdkColor      *a,
                              GdkColor      *b)
{
	GdkColor col;
	gint     dr, dg, db;
	gint     i;
	
  /* if b ommitted, draw solid with a */
  if (b == NULL) {
    /* unless a is also not specified */
    if (a)
      gdk_gc_set_rgb_fg_color(gc, a);

		gdk_draw_rectangle(drawable, gc, TRUE, x, y, width, height);
		return;
	}

  /* render gradient */
 	col= *a;
  if (horizontal) {
    /* calculate delta per line. this won't work at all for gradients larger than the colordifference */
  	dr= (b->red -a->red) / height;
  	dg= (b->green -a->green) / height;
  	db= (b->blue -a->blue) / height;

    /* draw it */
  	for (i= 0; i < height; i++) {
      gdk_gc_set_rgb_fg_color(gc, &col);
  		gdk_draw_line(drawable, gc, x, y +i, x +width -1, y +i);
  		
  		col.red+=   dr;
  		col.green+= dg;
  		col.blue+=  db;
  	}
  }
  else {
  	dr= (b->red -a->red) / width;
  	dg= (b->green -a->green) / width;
  	db= (b->blue -a->blue) / width;

  	for (i= 0; i < width; i++) {
      gdk_gc_set_rgb_fg_color(gc, &col);
  		gdk_draw_line(drawable, gc, x +i, y, x +i, y +height -1);
  		
  		col.red+=   dr;
  		col.green+= dg;
  		col.blue+=  db;
  	}
  }
}

/*
 * draw vertical arrow
 */
static void ac_draw_varrow( GdkWindow     *window,
                            GdkGC         *gc,
                            GtkShadowType  shadow_type,
                            GdkRectangle  *area,
                            GtkArrowType   arrow_type,
                            gint           x,
                            gint           y,
                            gint           width,
                            gint           height)
{
  gint steps, extra;
  gint y_start, y_increment;
  gint i;

  width= width +width % 2 -1;	/* Force odd */
  steps= 1 +width / 2;
  extra= height -steps;

  if (area)
    gdk_gc_set_clip_rectangle(gc, area);

  if (arrow_type == GTK_ARROW_DOWN) {
    y_start= y;
    y_increment= 1;
  }
  else {
    y_start= y + height - 1;
    y_increment= -1;
  }

  for (i = extra; i < height; i++)
    gdk_draw_line (window, gc,
		     x + (i - extra),              y_start + i * y_increment,
		     x + width - (i - extra) - 1,  y_start + i * y_increment);

  if (area)
    gdk_gc_set_clip_rectangle(gc, NULL);
}


/*
 * draw horizontal arrow
 */
static void ac_draw_harrow( GdkWindow     *window,
                            GdkGC         *gc,
                            GtkShadowType  shadow_type,
                            GdkRectangle  *area,
                            GtkArrowType   arrow_type,
                            gint           x,
                            gint           y,
                            gint           width,
                            gint           height)
{
  gint steps, extra;
  gint x_start, x_increment;
  gint i;

  height= height +height % 2 -1;	/* Force odd */
  steps= 1 +height / 2;
  extra= width -steps;

  if (arrow_type == GTK_ARROW_RIGHT) {
    x_start= x;
    x_increment= 1;
  }
  else {
    x_start= x +width -1;
    x_increment= -1;
  }

  for (i = extra; i < width; i++)
    gdk_draw_line (window, gc,
		     x_start + i * x_increment, y + (i - extra),
		     x_start + i * x_increment, y + height - (i - extra) - 1);
}


/*
 * swaps a colorindex, i.e. turns 0 to 8, 1 to 7...
 */
static inline gint csw(gboolean swap, gint color_id) {
  return swap ? NUM_SHADES -1 -color_id : color_id;
}


/*
 * draw a box with only very small rounded edges (i.e. for scrollbars)
 * bg1/bg2: background color indices from anachron style
 */
static void ac_draw_box_small(GdkDrawable   *window,
                              GtkStyle      *style,
                              GtkStateType   state_type,
                              gint           x,
                              gint           y,
                              gint           width,
                              gint           height,
                              gint           bg1,
                              gint           bg2,
                              gboolean       down,
                              gboolean       horizontal)
{
  GdkGC *gc=   ANACHRON_STYLE(style)->gc;
  gint         x1, x2, y1, y2;
  GdkColor    *palette;

  /* retrieve suitable pallette for current state */
  palette= get_state_palette(style, state_type);
  x1= x;
  y1= y;
  x2= x +width -1;
  y2= y +height -1;

  /* draw widget */
  ac_draw_gradient(window, gc, x +1, y +1, width -2, height -2, horizontal, &palette[bg1], &palette[bg2]);

  gdk_gc_set_rgb_fg_color(gc, &palette[csw(down, 0)]);
  gdk_draw_line( window, gc, x1 +1, y1, x2 -1, y1);   // top line
  gdk_draw_line( window, gc, x1, y1 +1, x1, y2 -2);   // left line

  gdk_gc_set_rgb_fg_color(gc, &palette[csw(down, 8)]);
  gdk_draw_line( window, gc, x2, y1 +2, x2, y2 -1);   // right line
  gdk_draw_line( window, gc, x1 +2, y2, x2 -1, y2);   // bottom line

  gdk_gc_set_rgb_fg_color(gc, &palette[csw(down, 2)]);
  gdk_draw_point(window, gc, x1,     y2 -1);          // bottom left3

  gdk_gc_set_rgb_fg_color(gc, &palette[csw(down, 3)]);
  gdk_draw_point(window, gc, x1,     y1);             // top left

  gdk_gc_set_rgb_fg_color(gc, &palette[csw(down, 6)]);
  gdk_draw_point(window, gc, x2,     y1);             // top right
  gdk_draw_point(window, gc, x1,     y2);             // bottom left2
  gdk_draw_point(window, gc, x2,     y2);             // bottom right

  gdk_gc_set_rgb_fg_color(gc, &palette[csw(down, 7)]);
  gdk_draw_point(window, gc, x2,     y1 +1);          // top right2
  gdk_draw_point(window, gc, x1 +1,  y2);             // bottom left1
}


/*
 * draws a larger box used for normal buttons
 */
static void ac_draw_box_large(GdkDrawable   *window,
                              GtkStyle      *style,
                              GtkStateType   state_type,
                              gint           x,
                              gint           y,
                              gint           width,
                              gint           height,
                              gint           bg1,
                              gint           bg2,
                              gboolean       down,
                              gboolean       horizontal)
{
  GdkGC *gc=   ANACHRON_STYLE(style)->gc;
  gint         x1, x2, y1, y2;
  GdkColor    *palette;

  /* retrieve suitable pallette for current state */
  palette= get_state_palette(style, state_type);
  x1= x;
  y1= y;
  x2= x +width -1;
  y2= y +height -1;

  /* draw widget */
  ac_draw_gradient(window, gc, x +1, y +1, width -2, height -2, horizontal, &palette[bg1], &palette[bg2]);

  gdk_gc_set_rgb_fg_color(gc, &palette[csw(down, 0)]);
  gdk_draw_line( window, gc, x1 +3, y1,    x2 -3, y1);    // top line
  gdk_draw_line( window, gc, x1,    y1 +3, x1,    y2 -3); // left line

  gdk_gc_set_rgb_fg_color(gc, &palette[csw(down, 8)]);
  gdk_draw_line( window, gc, x2,    y1 +3, x2,    y2 -3); // right line
  gdk_draw_line( window, gc, x1 +3, y2,    x2 -3, y2);    // bottom line

  gdk_gc_set_rgb_fg_color(gc, &palette[csw(down, 0)]);
  gdk_draw_point(window, gc, x1 +1, y1 +1);           // 1/1

  gdk_gc_set_rgb_fg_color(gc, &palette[csw(down, 1)]);
  gdk_draw_point(window, gc, x1 +2, y1);              // 2/0
  gdk_draw_point(window, gc, x1,    y1 +2);           // 0/2
  gdk_draw_point(window, gc, x1,    y2 -2);           // 0/4
  gdk_draw_point(window, gc, x2 -2, y1);              // 4/0

  gdk_gc_set_rgb_fg_color(gc, &palette[csw(down, 2)]);
  gdk_draw_point(window, gc, x1 +1, y1);              // 1/0
  gdk_draw_point(window, gc, x1,    y1 +1);           // 0/1
  gdk_draw_point(window, gc, x1 +2, y1 +1);           // 2/1
  gdk_draw_point(window, gc, x1 +1, y1 +2);           // 1/2

  gdk_gc_set_rgb_fg_color(gc, &palette[csw(down, 3)]);
  gdk_draw_point(window, gc, x1 +1, y2 -2);           // 1/4
  gdk_draw_point(window, gc, x1,    y2 -1);           // 0/5
  gdk_draw_point(window, gc, x2 -1, y1);              // 5/0
  gdk_draw_point(window, gc, x2 -2, y1 +1);           // 4/1

  gdk_gc_set_rgb_fg_color(gc, &palette[csw(down, 4)]);
  gdk_draw_point(window, gc, x1 +1, y2 -1);           // 1/5
  gdk_draw_point(window, gc, x1 +2, y2 -1);           // 2/5
  gdk_draw_point(window, gc, x1 +1, y2);              // 1/6
  gdk_draw_point(window, gc, x2 -1, y1 +1);           // 5/1
  gdk_draw_point(window, gc, x2,    y1 +1);           // 6/1
  gdk_draw_point(window, gc, x2 -1, y1 +2);           // 5/2

  gdk_gc_set_rgb_fg_color(gc, &palette[csw(down, 5)]);
  gdk_draw_point(window, gc, x1 +2, y2);              // 2/6
  gdk_draw_point(window, gc, x2,    y1 +2);           // 6/2

  gdk_gc_set_rgb_fg_color(gc, &palette[csw(down, 6)]);
  gdk_draw_point(window, gc, x2 -1, y2 -2);           // 5/4
  gdk_draw_point(window, gc, x2 -2, y2 -1);           // 4/5
  gdk_draw_point(window, gc, x2,    y2 -1);           // 6/5

  gdk_gc_set_rgb_fg_color(gc, &palette[csw(down, 7)]);
  gdk_draw_point(window, gc, x2 -2, y2);              // 4/6
  gdk_draw_point(window, gc, x2 -1, y2);              // 5/6

  gdk_gc_set_rgb_fg_color(gc, &palette[csw(down, 8)]);
  gdk_draw_point(window, gc, x2,    y2 -2);           // 6/4
  gdk_draw_point(window, gc, x2 -1, y2 -1);           // 5/5
}


/*
 * draw a little bump, used to decorate toolbar handles
 */
static void ac_draw_bump( GdkDrawable   *window,
                          GtkStyle      *style,
                          GtkStateType   state_type,
                          gint           x,
                          gint           y,
                          gint           length,
                          gboolean       horizontal)
{
  GdkGC     *gc= ANACHRON_STYLE(style)->gc;
  GdkColor  *palette;

  /* retrieve suitable pallette for current state */
  palette= get_state_palette(style, state_type);

  if (horizontal) {
    gdk_gc_set_rgb_fg_color(gc, &palette[7]);
    gdk_draw_point(window, gc, x, y);                           // top left
    gdk_gc_set_rgb_fg_color(gc, &palette[6]);
    gdk_draw_point(window, gc, x, y +1);                        // bottom left

    gdk_gc_set_rgb_fg_color(gc, &palette[8]);
    gdk_draw_line(window, gc, x +1, y, x +length -2, y);        // top
    gdk_gc_set_rgb_fg_color(gc, &palette[0]);
    gdk_draw_line(window, gc, x +1, y +1, x +length -2, y +1);  // bottom

    gdk_gc_set_rgb_fg_color(gc, &palette[2]);
    gdk_draw_point(window, gc, x +length -1, y);                // top right
    gdk_gc_set_rgb_fg_color(gc, &palette[1]);
    gdk_draw_point(window, gc, x +length -1, y +1);             // bottom right
  }
  else {
    // FIXME unimplemented
  }
}


/*
 * draw horizontal/vertical line
 */
static void ac_draw_line( GdkDrawable   *window,
                          GtkStyle      *style,
                          GtkStateType   state_type,
                          gint           x,
                          gint           y,
                          gint           length,
                          gboolean       horizontal)
{
  GdkGC *gc=   ANACHRON_STYLE(style)->gc;
  GdkColor    *palette;

  /* retrieve suitable pallette for current state */
  palette= get_state_palette(style, state_type);

  /* draw horizontal shadowed line */
  if (horizontal) {
    gdk_gc_set_rgb_fg_color(gc, &palette[7]);
    gdk_draw_line(window, gc, x, y, x +length -1, y);
    gdk_gc_set_rgb_fg_color(gc, &palette[2]);
    gdk_draw_line(window, gc, x +1, y +1, x +length, y +1);
  }

  /* vertical.. */
  else {
    gdk_gc_set_rgb_fg_color(gc, &palette[7]);
    gdk_draw_line(window, gc, x, y, x, y +length -1);
    gdk_gc_set_rgb_fg_color(gc, &palette[2]);
    gdk_draw_line(window, gc, x +1, y +1, x +1, y +length);
  }
}


/*
 * draw a tab header element
 * FIXME: implement bottom tabs, left tabs, right tabs
 */
static void ac_draw_notebook_tab( GdkDrawable   *window,
                                  GtkStyle      *style,
                                  GtkStateType   state_type,
                                  gint           x,
                                  gint           y,
                                  gint           width,
                                  gint           height,
                                  gboolean       down,
                                  gboolean       horizontal)
{
  GdkGC *gc=   ANACHRON_STYLE(style)->gc;
  gint         x1, x2, y1, y2, n1, n2, p1, p2;
  GdkColor    *palette;
  gint         gb;

  /* retrieve suitable pallette for current state */
  palette= get_state_palette(style, state_type);

  /* swap xy1/xy2 for pressed button, offsets nx (negative) and px (positive) also need to be swapped */
  if (down) {
    x1= x +width -1;
    y1= y +height -1;
    x2= x;
    y2= y;
    n1= 1; n2= 2; p1= -1; p2= -2;
  }
  else {
    x1= x;
    y1= y;
    x2= x +width -1;
    y2= y +height -1;
    n1= -1; n2= -2; p1= 1; p2= 2;
  }

  /* strong gradient for upper part, less strong below that */
  gb= (height -1) /4;
  ac_draw_gradient(window, gc, x +1, y +1,     width -2, gb,            horizontal, &palette[0], &palette[2]);
  ac_draw_gradient(window, gc, x +1, y +1 +gb, width -2, height -1 -gb, horizontal, &palette[2], &palette[4]);

  gdk_gc_set_rgb_fg_color(gc, &palette[0]);
  gdk_draw_line( window, gc, x1 +p1, y1, x2 +n1, y1);   // top line
  gdk_draw_line( window, gc, x1, y1 +p1, x1, y2 +n1);   // left line

  gdk_gc_set_rgb_fg_color(gc, &palette[8]);
  gdk_draw_line( window, gc, x2, y1 +p2, x2, y2 +n1);   // right line

//  if (state_type != GTK_STATE_SELECTED)
//    gdk_draw_line( window, gc, x1 +p2, y2, x2 +n1, y2);   // bottom line

//  gdk_gc_set_rgb_fg_color(gc, &palette[1]);
//  gdk_draw_point(window, gc, x1,     y2 +n1);           // bottom left3

  gdk_gc_set_rgb_fg_color(gc, &palette[3]);
  gdk_draw_point(window, gc, x1,     y1);               // top left

  gdk_gc_set_rgb_fg_color(gc, &palette[5]);
  gdk_draw_point(window, gc, x2,     y1);               // top right

  gdk_gc_set_rgb_fg_color(gc, &palette[1]);
  gdk_draw_point(window, gc, x1,     y2);               // bottom left2

  gdk_gc_set_rgb_fg_color(gc, &palette[6]);
  gdk_draw_point(window, gc, x2,     y1 +p1);           // top right2
  gdk_draw_point(window, gc, x2,     y2);               // bottom right

//  gdk_gc_set_rgb_fg_color(gc, &palette[8]);
//  gdk_draw_point(window, gc, x1 +p1, y2);               // bottom left1
}












static void
draw_part (GdkDrawable  *drawable,
	   GdkGC        *gc,
	   GdkRectangle *area,
	   gint          x,
	   gint          y,
	   Part          part)
{
  if (area)
    gdk_gc_set_clip_rectangle (gc, area);
  
  if (!parts[part].bmap)
      parts[part].bmap = gdk_bitmap_create_from_data (drawable,
						      parts[part].bits,
						      PART_SIZE, PART_SIZE);

  gdk_gc_set_ts_origin (gc, x, y);
  gdk_gc_set_stipple (gc, parts[part].bmap);
  gdk_gc_set_fill (gc, GDK_STIPPLED);

  gdk_draw_rectangle (drawable, gc, TRUE, x, y, PART_SIZE, PART_SIZE);

  gdk_gc_set_fill (gc, GDK_SOLID);

  if (area)
    gdk_gc_set_clip_rectangle (gc, NULL);
}

static void
draw_check(GtkStyle      *style,
	   GdkWindow     *window,
	   GtkStateType   state,
	   GtkShadowType  shadow,
	   GdkRectangle  *area,
	   GtkWidget     *widget,
	   const gchar   *detail,
	   gint           x,
	   gint           y,
	   gint           width,
	   gint           height)
{
  x -= (1 + PART_SIZE - width) / 2;
  y -= (1 + PART_SIZE - height) / 2;
      
  if (detail && strcmp (detail, "check") == 0)	/* Menu item */
    {
      if (shadow == GTK_SHADOW_IN)
	{
	  draw_part (window, style->black_gc, area, x, y, CHECK_TEXT);
	  draw_part (window, style->dark_gc[state], area, x, y, CHECK_AA);
	}
    }
  else
    {
      draw_part (window, style->black_gc, area, x, y, CHECK_BLACK);
      draw_part (window, style->dark_gc[state], area, x, y, CHECK_DARK);
      draw_part (window, style->mid_gc[state], area, x, y, CHECK_MID);
      draw_part (window, style->light_gc[state], area, x, y, CHECK_LIGHT);
      draw_part (window, style->base_gc[state], area, x, y, CHECK_BASE);
      
      if (shadow == GTK_SHADOW_IN)
	{
	  draw_part (window, style->text_gc[state], area, x, y, CHECK_TEXT);
	  draw_part (window, style->text_aa_gc[state], area, x, y, CHECK_AA);
	}
    }
}

static void
draw_option(GtkStyle      *style,
	    GdkWindow     *window,
	    GtkStateType   state,
	    GtkShadowType  shadow,
	    GdkRectangle  *area,
	    GtkWidget     *widget,
	    const gchar   *detail,
	    gint           x,
	    gint           y,
	    gint           width,
	    gint           height)
{
  x -= (1 + PART_SIZE - width) / 2;
  y -= (1 + PART_SIZE - height) / 2;
      
  if (detail && strcmp (detail, "option") == 0)	/* Menu item */
    {
      if (shadow == GTK_SHADOW_IN)
	draw_part (window, style->fg_gc[state], area, x, y, RADIO_TEXT);
    }
  else
    {
      draw_part (window, style->black_gc, area, x, y, RADIO_BLACK);
      draw_part (window, style->dark_gc[state], area, x, y, RADIO_DARK);
      draw_part (window, style->mid_gc[state], area, x, y, RADIO_MID);
      draw_part (window, style->light_gc[state], area, x, y, RADIO_LIGHT);
      draw_part (window, style->base_gc[state], area, x, y, RADIO_BASE);
      
      if (shadow == GTK_SHADOW_IN)
	draw_part (window, style->text_gc[state], area, x, y, RADIO_TEXT);
    }
}


static void
option_menu_get_props (GtkWidget      *widget,
		       GtkRequisition *indicator_size,
		       GtkBorder      *indicator_spacing)
{
  GtkRequisition *tmp_size = NULL;
  GtkBorder *tmp_spacing = NULL;
  
  if (widget)
    gtk_widget_style_get (widget, 
			  "indicator_size", &tmp_size,
			  "indicator_spacing", &tmp_spacing,
			  NULL);

  if (tmp_size)
    {
      *indicator_size = *tmp_size;
      g_free (tmp_size);
    }
  else
    *indicator_size = default_option_indicator_size;

  if (tmp_spacing)
    {
      *indicator_spacing = *tmp_spacing;
      g_free (tmp_spacing);
    }
  else
    *indicator_spacing = default_option_indicator_spacing;
}

